@@ -1,15 +1,15 @@
 (require 'ert)
 (require 'elisp-refs)
 
-;; For Travis CI is recursing more deeply, meaning we hit recursion
+;; Tests in GitHub actions are recursing more deeply, meaning we hit recursion
 ;; limits. I suspect this is due to undercover collecting coverage
 ;; metrics.
-(when (getenv "TRAVIS")
+(when (getenv "CI")
   (message "Updating recursion limits from: max-specpdl-size: %s max-lisp-eval-depth: %s "
            max-specpdl-size
            max-lisp-eval-depth)
-  (setq max-specpdl-size 2500)
-  (setq max-lisp-eval-depth 1000))
+  (setq max-specpdl-size 4000)
+  (setq max-lisp-eval-depth 2000))
 
 (defmacro with-temp-backed-buffer (contents &rest body)
   "Create a temporary file with CONTENTS, and evaluate BODY
@@ -269,7 +269,7 @@ backquote forms."
      (should
       (equal (length calls) 2)))))
 
-(ert-deftest elisp-refs--find-calls-let-body ()
+(ert-deftest elisp-refs--find-macro-calls-let-body ()
   "We should find macro calls in let body."
   (with-temp-backed-buffer
    "(let (bar) (foo)) (let* (bar) (foo))"
